import numpy as np
import math
import datetime

def readMAPSFile(filename, bandWidth, intTime, SEFD, sdevAmpPer, phaseErrBool):

	u,v,amp,phase,year,day,hour,minute,second,ant1,ant2,time = extractData(filename);

	# u and v are in kilowavelengths, put them in wavelengths
	kilo = 1000.0;
	u = u*kilo;
	v = v*kilo;

	# compute time of each sample
	timeFlt = (day*86400.0) + (hour*3600.0) + (minute*60.0) + (second) - (day[0]*86400.0);
	mjd_offset = 54100.0 + day + (365.0*(year-2007.0)) + map(math.floor, (year-2007.0)/4.0);
	mjd = (timeFlt/86400.0) + mjd_offset;
    
	# get error of each sample
	viserror  = np.array(map( math.sqrt, SEFD[ant1-1]*SEFD[ant2-1] / (2.0 * bandWidth*intTime) ));	
	#viserror  = (1./0.88)* np.array(map( math.sqrt, SEFD[ant1-1]*SEFD[ant2-1] / (2.0 * bandWidth*intTime) ));	
	
	amp, phase, viserror = addNoise(amp, phase, ant1, ant2, mjd, viserror, sdevAmpPer, phaseErrBool); 
	
	return u, v, amp, phase, ant1, ant2, viserror, mjd, time

	# compute closure data
    	#t3amp, t3amperr, t3phi, t3phierr, uClosure, vClosure, antOrder, timeClosure = getClosures(u,v,mjd,ant1,ant2,viserror,time,amp,phase);
	
	
def addNoise(amp, phase, ant1, ant2, mjd, viserror, sdevAmpPer, phaseErrBool):
	
	maxant  = np.max([ np.max(ant2),  np.max(ant1)])
	uniqueMjd = np.unique(mjd); 

	if phaseErrBool:
		randMtx = np.random.uniform(low=0, high=360, size=(maxant+1, len(uniqueMjd)) );
	else:
		randMtx = np.zeros((maxant+1, len(uniqueMjd))); 
	
	factor = (sdevAmpPer/100.0);
	if sdevAmpPer>0:
		randAmpMtx = np.random.normal(1, factor, (maxant+1, len(uniqueMjd))); 
	else:
		randAmpMtx = np.ones((maxant+1, len(uniqueMjd))); 
	
	
	for i in range(0, len(phase)):
		index = np.where(uniqueMjd==mjd[i])[0][0]
		
		#add phase error
		phase[i] = phase[i] + randMtx[ant1[i]][index] - randMtx[ant2[i]][index]; 
		
		#add a gain error as a percentage of the amplitude
		amp[i] = randAmpMtx[ant1[i]][index] * randAmpMtx[ant2[i]][index] * amp[i]; 
		viserror[i] = np.sqrt( viserror[i]**2 + (amp[i]**2 * (2 * factor**2) ) ); 
	
	return amp, phase, viserror

def getClosures(u,v,mjd,ant1,ant2,viserror,time,amp,phase):
    
	sign_convention = -1;
	mjd_unique = np.unique(mjd);
    
	t3amp =[];
	t3amperr = [];
	t3phi = [];
	t3phierr = [];
	uClosure = [];
	vClosure = [];
	antOrder = [];
	timeClosure = [];
	mjdClosure = []; 
	ampClosures = []; 
	errClosures = []; 
    
	for mjd_curr in mjd_unique:
        
		valid = (mjd == mjd_curr);
		numVis = sum(valid);
        
		if (numVis >= 3):
            
			uTmp = u[valid]
			vTmp = v[valid]
			ant1Tmp = ant1[valid]
			ant2Tmp = ant2[valid]
			errTmp = viserror[valid]
			timeTmp = time[valid]
			ampTmp = amp[valid]
			phaseTmp = phase[valid]
           			
			for i in range (0, numVis-2):
				for j in range (i+1, numVis-1):
					for k in range (j+1, numVis):
						if ((ant1Tmp[i]==ant1Tmp[j]) and (ant2Tmp[i]==ant1Tmp[k]) and (ant2Tmp[j]==ant2Tmp[k])):
                            
							# baseline goes ant1[i] ant2[i] ant2[k] and then from ant1[i] to ant2[k]
							uClosure.append( np.array( [ uTmp[i], uTmp[k] ] ) ); #u1 u2
							vClosure.append( np.array( [ vTmp[i], vTmp[k] ] ) ); #v1 v2
							antOrder.append( np.array( [ ant1Tmp[i], ant2Tmp[i], ant2Tmp[k] ] ) ); #v1 v2
                            
							timeClosure.append( timeTmp[i] );
							mjdClosure.append( mjd_curr );
                            
							t3amp.append( ampTmp[i]*ampTmp[j]*ampTmp[k] );
                            				valt3amperr = math.sqrt( math.pow(ampTmp[j]*ampTmp[k]*errTmp[i], 2) + math.pow(ampTmp[i]*ampTmp[k]*errTmp[j], 2) + math.pow(ampTmp[i]*ampTmp[j]*errTmp[k], 2) )
							t3amperr.append( valt3amperr );
                            
							ampClosures.append( np.array( [ampTmp[i], ampTmp[j], ampTmp[k] ] ) );
							errClosures.append( np.array( [errTmp[i], errTmp[j], errTmp[k] ] ) ); 

							closureAng = (phaseTmp[i]-phaseTmp[j]+phaseTmp[k])*sign_convention;
							while (closureAng>180.0):
								closureAng = closureAng - 360.0;
							while (closureAng<=-180.0):
								closureAng = closureAng + 360.0;
							
							t3phi.append( closureAng );
							t3phierr.append( 180.0/math.pi * valt3amperr / (ampTmp[i]*ampTmp[j]*ampTmp[k]) );
                                        
	return (t3amp, t3amperr, t3phi, t3phierr, uClosure, vClosure, antOrder, timeClosure, mjdClosure, ampClosures, errClosures)

                                        
def extractData(filename):
                      
	#read file
	lines = open(filename, "r").read().splitlines()
                                        
	#remove bad lines
	lines[:] = [x for x in lines if x != 'UV data taken from visgen.']
	lines[:] = [x for x in lines if x != 'Scan Start (UT)            U(klam)      V(klam)      W(klam)  Baseline   Channel         Visibility (amp, phase)   Weight      Sigma']

	# parse the lines
	u = np.array([float(x.split()[1]) for x in lines])
	v = np.array([float(x.split()[2]) for x in lines])
	w = np.array([float(x.split()[3]) for x in lines])

	amp = [x.split()[7] for x in lines]
	amp = np.array([float(x.split(',')[0]) for x in amp])
	phase = [x.split()[8] for x in lines]
	phase = np.array([float(x.split(')')[0]) for x in phase])

	time = np.array([x.split()[0] for x in lines])
	year = np.array([int(x.split(':')[0]) for x in time])
	day = np.array([int(x.split(':')[1]) for x in time])
	hour = np.array([int(x.split(':')[2]) for x in time])
	minute = np.array([int(x.split(':')[3]) for x in time])
	second = np.array([float(x.split(':')[4]) for x in time])
	
	dtTime = [];
	for i in range(0,len(year)):
		microsec = int(math.floor(0.5 + (second[i] - float(math.floor(second[i])))*1000000.0));
		dtTime.append( datetime.datetime.strptime(time[i].split('.')[0], '%Y:%j:%H:%M:%S') + datetime.timedelta(0,0,microsec) )
	dtTime = np.array(dtTime);

	ants = [x.split()[4] for x in lines]
	ant1 = np.array([int(x.split('-')[0]) for x in ants])
	ant2 = np.array([int(x.split('-')[1]) for x in ants])

	return (u,v,amp,phase,year,day,hour,minute,second,ant1,ant2,dtTime)
