import readUVdata
import oifits
import writeData
import RADECConv
import numpy as np

# sample parameters
MHZ2HZ = 1000000.0
frequency = 227297 * MHZ2HZ
bandWidth = 4096 * MHZ2HZ
intTime = 10

# convert the center of the FOV to degrees
RACentString = '12:30:49.423382'
DECCentString = '12:23:28.04366'
RACent = RADECConv.convHMS(RACentString);
DECCent = RADECConv.convDMS(DECCentString);

# telescope info
antennaNames = np.array(['Phased Mauna Kea','SMT (Arizona)','CARMA (CA)','LMT (Mexico)', 'Phased ALMA', 'IRAM 30m (PV)','Plateau de Bure','South Pole Tel','GLT']);
SEFD = np.array([4900., 11900., 3500., 560., 110., 2900., 1600., 7300., 4744.])
antennaDiam = np.array([20.8,10,26.9,50,84.7,30,36.7,12,12]); 
antennaX = np.array([ -5464523.400,  -1828796.200,  -2397431.300,  -768713.9637, 2225037.1851, 5088967.9000,  4523998.40, 0.0, 1500692.0]); 
antennaY = np.array([ -2493147.080,  -5054406.800,  -4482018.900, -5988541.7982, -5441199.1620, -301681.6000,  468045.240, 0.0, -1191735.0]); 
antennaZ = np.array([ 2150611.750,  3427865.200,  3843524.500, 2063275.9472, -2479303.4629, 3825015.8000,  4460309.760, -6359587.3, 6066409.0]); 


# input and output file names
uvFile = 'UVdata.txt'
oifitsFile = 'oifitsData.oifits'

phaseErrorBool = True;
sdevAmpPer = 5;

# get visibilities from uvdata file
u, v, amp, phase, ant1, ant2, viserror, mjd, time = readUVdata.readMAPSFile(uvFile, bandWidth, intTime, SEFD, sdevAmpPer, phaseErrorBool);

# get the closure data
t3amp, t3amperr, t3phi, t3phierr, uClosure, vClosure, antOrder, timeClosure, mjdClosure, ampClosures, errClosures = readUVdata.getClosures(u,v,mjd,ant1,ant2,viserror,time,amp,phase);
 	
# write data to oifits file
writeData.writeOIFITS(oifitsFile, RACent, DECCent, frequency, bandWidth, intTime, amp, viserror, phase, 
	viserror, u, v, ant1, ant2, time, t3amp, t3amperr, t3phi, t3phierr, uClosure, vClosure, antOrder, timeClosure, 
	antennaNames, antennaDiam, antennaX, antennaY, antennaZ)
